;------------------------------
; Marble Craze bank 2
;------------------------------
; Includes:
;
; - VBlank/game logic
;------------------------------


	org $2000
	rorg $1000


; READS LEVELS ARRAY (could be moved)
;------------------------------------------------------------------------
; getLevelPointer
;------------------------------------------------------------------------
; Get pointer to the level data structure
;------------------------------------------------------------------------
getLevelPointer

	; Get level pointer	
	lda level
	and #%00011111
	asl
	tay
	lda levels,y
	sta levelL
	lda levels+1,y
	sta levelH

	rts






; CANNOT BE MOVED
;--------------------------------------------------------------------------
; processGameLogic
;--------------------------------------------------------------------------
; calls game logic routines.  This routine is called from Overscan.
;--------------------------------------------------------------------------
processGameLogic

	; ---------------------------------
	; Turn on/off music using BW/Color switch

	lda SWCHB
	and #8
	beq noSong
	jmp callSong1

noSong
	lda #0
	sta AUDV0
	sta AUDV1

rtnCallSong1
	;--------------------------------------------




	;-------------------------------- Game Logic Player 1

	; if at end of level or end of game, skip game logic
	lda level
	and #%11000000
	bne rtnGameLogicA

	jsr getLevelPointer

	; player 1
	ldx #0
	jmp gameLogic
rtnGameLogicA
	;-------------------------------------------------------



skipGameLogic
	jmp rtnProcessGameLogic


; CANNOT BE MOVED (song player can be moved, though)
;--------------------------------------------------------------------------
; VBlank2
;--------------------------------------------------------------------------
; Additional VBlank bank 2 VBlank code
;--------------------------------------------------------------------------
VBlank2



	; --------------------- Game logic player 2

	; if at end of level or end of game, skip game logic
	lda level
	and #%11000000
	bne rtnGameLogicB

	; player 2
	ldx #1
	jmp gameLogic
rtnGameLogicB
	;----------------------------------------------



	;-----------------------------------  Load screen data

	; Alternate loading the left and right players to reduce cycles
	lda frame
	and #1
	tax

	jsr loadScreen
	;-------------------------------------


endVBlank2

	; return from VBlank2
	jmp rtnVBlank2




; HAS UP TO 4 SCREEN READS.   READS LEVEL DATA, COLORS, and calls CHECKPF
;--------------------------------------------------------------------------
; gameLogic
;--------------------------------------------------------------------------
; Handles general game logic for a player.
; 
; Preload:
; X: Player (0 or 1)
;
; temp variables:
; 	screenL/screenH: control pointer
;	pMarble: 		"screen"
; 	pMarble+1: 		"player"
;	temp5: 			"status"
;	temp16L: 		finish color
;	temp16H:		reset time/general temp
;	pMarble2:		old y coord
; 	pMarble2+1:		old x coord
;--------------------------------------------------------------------------
gameLogic

	stx player
	
	; move status to general status variable
	lda p1Status,x
	sta status

	; No need for game logic if inactive
	cmp #INACTIVE
	bne continueLogic

	jmp endLogic

continueLogic

	; Save the X and Y coordinates in case
	; the "walls" check needs to override the paddle control
	lda p1x,x
	sta pMarble2
	lda p1y,x
	sta pMarble2+1

	; move screen to general status variable
	lda p1Screen,x
	sta screen

	; Get screen pointer
	
	; remove walls/falling flags
	and #%00011111
	tay
	lda screenPtrOffset,y
	clc
	adc #4
	tay
	lda (levelL),y
	sta screenL
	iny
	lda (levelL),y
	sta screenH

	; get reset time
	ldy #5
	lda (levelL),y
	sta temp16H

	; get finish color
	ldy #2
	lda (levelL),y
	sta temp16L

	jmp callGameLogic2


rtnCallGameLogic2
endLogic

	cpx #1
	beq return2

	jmp rtnGameLogicA

return2
	jmp rtnGameLogicB





 
;------------------------------------------------------------------------
; loadScreen
;------------------------------------------------------------------------
; load the playfield data for player 1 and/or 2 as needed
;
; Preload:
; X: player to process (0 or 1)
;
; uses these temps:
;   screenL/screenH 	: pointer to pf data
;   temp/temp2			: pointer to color data
;   pMarble2,pMarble2+1	: pointer to control data
;------------------------------------------------------------------------
loadScreen

	lda p1Status,x
	cmp #INACTIVE
	beq goInactive

	; Skip loadScreen if GAMEOVER or NEXTLEVEL flags are set
	lda level
	and #%11000000
	bne noLoad1

	lda p1Status,x
	cmp #LOADREADY
	beq load1
	cmp #RESET
	beq load2
	cmp #STARTUP
	beq load2

	cmp #SCREENLOAD
	bne noLoad1

	lda #LOADREADY
	sta p1Status,x

noLoad1
	rts

	;------------------- inactive screen setup
goInactive
	lda #0
	sta p1TimeL,x
	sta p1TimeH,x
	sta p1Lives,x
	sta pwr1x,x
	sta pwr1y,x
	
	lda #162
	sta p1y,x


	;**************
	; LOAD SCREEN 2
	;**************

	jmp callLoadScreen2


	;--------------------- normal screen setup
load1
	lda #NORMAL
	sta p1Status,x
load2

	; Get screen pointer
	lda p1Screen,x
	
	; Get all the pointers
	and #%00011111
	tay
	lda screenPtrOffset,y
	tay

	; playfield pointer
	lda (levelL),y
	sta screenL
	iny
	lda (levelL),y
	sta screenH
	iny 

	; color pointer
	lda (levelL),y
	sta temp
	iny 
	lda (levelL),y
	sta temp2
	iny 
	
	; control pointer
	lda (levelL),y
	sta pMarble2
	iny 
	lda (levelL),y
	sta pMarble2+1
	iny 

	; Get colors
	ldy #0
	lda (levelL),y			;;;;;;;;;;;;;;; LEVEL SETUP READ
	sta bkColor
	iny
	lda (levelL),y
	sta pfColor
	
	;**************
	; LOAD SCREEN 2
	;**************
	
	jmp callLoadScreen2

rtnCallLoadScreen2
	rts













screenPtrOffset
	byte 6,12,18,24,30,36,42,48,54,60,66,72,78,84





; This isn't really a level, it's data for the 
; inactive player animation




level1
	byte $00		; bk color
	byte $06		; pf color
	byte $A3		; finish color
	byte $00		; slope up color
	byte $00		; slope down color
	byte $55		; time

	word lev10, col10, cont10
	word lev11, col11, cont11
	word lev12, col12, cont12
	word lev13, col13, cont13
	word lev14, col14, cont14
	word lev15, col15, cont15
	word lev16, col16, cont16
	word lev17, col17, cont17



level2
	byte $44		; bk color
	byte $00		; pf color
	byte $45		; finish color
	byte $00		; slope up color
	byte $00		; slope down color
	byte $55		; time

	word lev20, col20, cont20
	word lev21, col21, cont21
	word lev22, col22, cont22
	word lev23, col23, cont23
	word lev24, col24, cont24
	word lev25, col25, cont25
	word lev26, col26, cont26
	word lev27, col27, cont27
	word lev28, col28, cont28
	word lev29, col29, cont29



levels
	word level1, level2, level2

	; MOVE








;--------------------------------------------------------------------------
; processGameLogic caller
;--------------------------------------------------------------------------
; call function then switch back to bank 1.
;--------------------------------------------------------------------------

	org $2FA3
	rorg $1FA3

	jmp processGameLogic
rtnProcessGameLogic
	ldx BANK1  		;(Switch back to bank 1)        


;--------------------------------------------------------------------------
; callSong1
;--------------------------------------------------------------------------
; Call to bank 4
;--------------------------------------------------------------------------

	org $2FB0
	rorg $1FB0

callSong1
	stx BANK4	
	nop         
	nop         
	nop         
	nop         
	nop         
	nop         
	jmp rtnCallSong1


;--------------------------------------------------------------------------
; callGameLogic2
;--------------------------------------------------------------------------
; Call to bank 5
;--------------------------------------------------------------------------

	org $2FC0
	rorg $1FC0

callGameLogic2
	stx BANK5	
	nop         
	nop         
	nop         
	nop         
	nop         
	nop         
	jmp rtnCallGameLogic2


;--------------------------------------------------------------------------
; callLoadScreen2
;--------------------------------------------------------------------------
; Call to bank 5
;--------------------------------------------------------------------------

	org $2FD0
	rorg $1FD0

callLoadScreen2
	stx BANK5	
	nop         
	nop         
	nop         
	nop         
	nop         
	nop         
	jmp rtnCallLoadScreen2


;--------------------------------------------------------------------------
; VBlank2 caller
;--------------------------------------------------------------------------
; call function then switch back to bank 1.
;--------------------------------------------------------------------------

	org $2FE3
	rorg $1FE3

	jmp VBlank2
rtnVBlank2
	ldx BANK1 


;--------------------------------------------------------------------------
; The cart may start up in this bank.  Make sure it switches
; back to bank 1 on startup.
;--------------------------------------------------------------------------
	org $2FED
	rorg $1FED

	lda BANK1

	org $2FFC	; Program startup vector
	.word $1FED
	.word $1FED

